@@ -1,19 +1,48 @@
 import mimetypes
 import pyrax
 import re
-import swiftclient
 from gzip import GzipFile
-from StringIO import StringIO
+
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 from django.core.files.base import File, ContentFile
 from django.core.files.storage import Storage
 
+from cumulus.authentication import Auth
 from cumulus.settings import CUMULUS
 
 
 HEADER_PATTERNS = tuple((re.compile(p), h) for p, h in CUMULUS.get("HEADERS", {}))
 
 
+def get_content_type(name, content):
+    """
+    Checks if the content_type is already set.
+    Otherwise uses the mimetypes library to guess.
+    """
+    if hasattr(content, "content_type"):
+        content_type = content.content_type
+    else:
+        mime_type, encoding = mimetypes.guess_type(name)
+        content_type = mime_type
+    return content_type
+
+
+def get_headers(name, content_type):
+    headers = {"Content-Type": content_type}
+    # gzip the file if its of the right content type
+    if content_type in CUMULUS.get("GZIP_CONTENT_TYPES", []):
+        headers["Content-Encoding"] = "gzip"
+    if CUMULUS["HEADERS"]:
+        for pattern, pattern_headers in HEADER_PATTERNS:
+            if pattern.match(name):
+                headers.update(pattern_headers.copy())
+    return headers
+
+
 def sync_headers(cloud_obj, headers={}, header_patterns=HEADER_PATTERNS):
     """
     Overwrites the given cloud_obj's headers with the ones given as ``headers`
@@ -48,128 +77,17 @@ def get_gzipped_contents(input_file):
     return ContentFile(zbuf.getvalue())
 
 
-class SwiftclientStorage(Storage):
+class SwiftclientStorage(Auth, Storage):
     """
     Custom storage for Swiftclient.
     """
     default_quick_listdir = True
-    api_key = CUMULUS["API_KEY"]
-    auth_url = CUMULUS["AUTH_URL"]
-    region = CUMULUS["REGION"]
-    connection_kwargs = {}
     container_name = CUMULUS["CONTAINER"]
-    use_snet = CUMULUS["SERVICENET"]
-    username = CUMULUS["USERNAME"]
+    container_uri = CUMULUS["CONTAINER_URI"]
+    container_ssl_uri = CUMULUS["CONTAINER_SSL_URI"]
     ttl = CUMULUS["TTL"]
+    file_ttl = CUMULUS["FILE_TTL"]
     use_ssl = CUMULUS["USE_SSL"]
-    use_pyrax = CUMULUS["USE_PYRAX"]
-
-    def __init__(self, username=None, api_key=None, container=None,
-                 connection_kwargs=None, container_uri=None):
-        """
-        Initializes the settings for the connection and container.
-        """
-        if username is not None:
-            self.username = username
-        if api_key is not None:
-            self.api_key = api_key
-        if container is not None:
-            self.container_name = container
-        if connection_kwargs is not None:
-            self.connection_kwargs = connection_kwargs
-        # connect
-        if CUMULUS["USE_PYRAX"]:
-            if CUMULUS["PYRAX_IDENTITY_TYPE"]:
-                pyrax.set_setting("identity_type", CUMULUS["PYRAX_IDENTITY_TYPE"])
-            if CUMULUS["AUTH_URL"]:
-                pyrax.set_setting("auth_endpoint", CUMULUS["AUTH_URL"])
-            if CUMULUS["AUTH_TENANT_ID"]:
-                pyrax.set_setting("tenant_id", CUMULUS["AUTH_TENANT_ID"])
-
-            pyrax.set_credentials(self.username, self.api_key)
-
-    def __getstate__(self):
-        """
-        Return a picklable representation of the storage.
-        """
-        return {
-            "username": self.username,
-            "api_key": self.api_key,
-            "container_name": self.container_name,
-            "use_snet": self.use_snet,
-            "connection_kwargs": self.connection_kwargs
-        }
-
-    def _get_connection(self):
-        if not hasattr(self, "_connection"):
-            if CUMULUS["USE_PYRAX"]:
-                public = not self.use_snet  # invert
-                self._connection = pyrax.connect_to_cloudfiles(region=self.region,
-                                                               public=public)
-            else:
-                self._connection = swiftclient.Connection(
-                    authurl=CUMULUS["AUTH_URL"],
-                    user=CUMULUS["USERNAME"],
-                    key=CUMULUS["API_KEY"],
-                    snet=CUMULUS["SERVICENET"],
-                    auth_version=CUMULUS["AUTH_VERSION"],
-                    tenant_name=CUMULUS["AUTH_TENANT_NAME"],
-                )
-        return self._connection
-
-    def _set_connection(self, value):
-        self._connection = value
-
-    connection = property(_get_connection, _set_connection)
-
-    def _get_container(self):
-        """
-        Gets or creates the container.
-        """
-        if not hasattr(self, "_container"):
-            if CUMULUS["USE_PYRAX"]:
-                self._container = self.connection.create_container(self.container_name)
-            else:
-                self._container = None
-        return self._container
-
-    def _set_container(self, container):
-        """
-        Sets the container (and, if needed, the configured TTL on it), making
-        the container publicly available.
-        """
-        if CUMULUS["USE_PYRAX"]:
-            if container.cdn_ttl != self.ttl or not container.cdn_enabled:
-                container.make_public(ttl=self.ttl)
-            if hasattr(self, "_container_public_uri"):
-                delattr(self, "_container_public_uri")
-        self._container = container
-
-    container = property(_get_container, _set_container)
-
-    def _get_container_url(self):
-        if self.use_ssl and CUMULUS["CONTAINER_SSL_URI"]:
-            self._container_public_uri = CUMULUS["CONTAINER_SSL_URI"]
-        elif self.use_ssl:
-            self._container_public_uri = self.container.cdn_ssl_uri
-        elif CUMULUS["CONTAINER_URI"]:
-            self._container_public_uri = CUMULUS["CONTAINER_URI"]
-        else:
-            self._container_public_uri = self.container.cdn_uri
-        if CUMULUS["CNAMES"] and self._container_public_uri in CUMULUS["CNAMES"]:
-            self._container_public_uri = CUMULUS["CNAMES"][self._container_public_uri]
-        return self._container_public_uri
-
-    container_url = property(_get_container_url)
-
-    def _get_object(self, name):
-        """
-        Helper function to retrieve the requested Object.
-        """
-        if name not in self.container.get_object_names():
-            return False
-        else:
-            return self.container.get_object(name)
 
     def _open(self, name, mode="rb"):
         """
@@ -182,32 +100,24 @@ class SwiftclientStorage(Storage):
         Uses the Swiftclient service to write ``content`` to a remote
         file (called ``name``).
         """
-        # Checks if the content_type is already set.
-        # Otherwise uses the mimetypes library to guess.
-        if hasattr(content.file, "content_type"):
-            content_type = content.file.content_type
-        else:
-            mime_type, encoding = mimetypes.guess_type(name)
-            content_type = mime_type
+        content_type = get_content_type(name, content.file)
+        headers = get_headers(name, content_type)
 
-        headers = {"Content-Type": content_type}
-
-        # gzip the file if its of the right content type
-        if content_type in CUMULUS.get("GZIP_CONTENT_TYPES", []):
-            content_encoding = headers["Content-Encoding"] = "gzip"
-        else:
-            content_encoding = None
-
-        if CUMULUS["USE_PYRAX"]:
-            # TODO set headers
-            if content_encoding == "gzip":
+        if self.use_pyrax:
+            if headers.get("Content-Encoding") == "gzip":
                 content = get_gzipped_contents(content)
             self.connection.store_object(container=self.container_name,
                                          obj_name=name,
                                          data=content.read(),
                                          content_type=content_type,
-                                         content_encoding=content_encoding,
+                                         content_encoding=headers.get("Content-Encoding", None),
+                                         ttl=self.file_ttl,
                                          etag=None)
+            # set headers/object metadata
+            self.connection.set_object_metadata(container=self.container_name,
+                                                obj=name,
+                                                metadata=headers,
+                                                prefix='')
         else:
             # TODO gzipped content when using swift client
             self.connection.put_object(self.container_name, name,
@@ -303,6 +213,8 @@ class SwiftclientStaticStorage(SwiftclientStorage):
     STATICFILES_STORAGE = "cumulus.storage.SwiftclientStaticStorage".
     """
     container_name = CUMULUS["STATIC_CONTAINER"]
+    container_uri = CUMULUS["STATIC_CONTAINER_URI"]
+    container_ssl_uri = CUMULUS["STATIC_CONTAINER_SSL_URI"]
 
 
 class SwiftclientStorageFile(File):
@@ -311,6 +223,7 @@ class SwiftclientStorageFile(File):
     def __init__(self, storage, name, *args, **kwargs):
         self._storage = storage
         self._pos = 0
+        self._chunks = None
         super(SwiftclientStorageFile, self).__init__(file=None, name=name,
                                                      *args, **kwargs)
 
@@ -342,24 +255,36 @@ class SwiftclientStorageFile(File):
 
     file = property(_get_file, _set_file)
 
-    def read(self, chunk_size=-1):
+    def read(self, chunk_size=None):
         """
         Reads specified chunk_size or the whole file if chunk_size is None.
 
         If reading the whole file and the content-encoding is gzip, also
         gunzip the read content.
+
+        If chunk_size is provided, the same chunk_size will be used in all
+        further read() calls until the file is reopened or seek() is called.
         """
-        if self._pos == self._get_size() or chunk_size == 0:
+        if self._pos >= self._get_size() or chunk_size == 0:
             return ""
 
-        if chunk_size < 0:
+        if chunk_size is None and self._chunks is None:
             meta, data = self.file.get(include_meta=True)
             if meta.get("content-encoding", None) == "gzip":
                 zbuf = StringIO(data)
                 zfile = GzipFile(mode="rb", fileobj=zbuf)
                 data = zfile.read()
         else:
-            data = self.file.get(chunk_size=chunk_size).next()
+            if self._chunks is None:
+                # When reading by chunks, we're supposed to read the whole file
+                # before calling get() again.
+                self._chunks = self.file.get(chunk_size=chunk_size)
+
+            try:
+                data = self._chunks.next()
+            except StopIteration:
+                data = ""
+
         self._pos += len(data)
         return data
 
@@ -376,9 +301,11 @@ class SwiftclientStorageFile(File):
         Opens the cloud file object.
         """
         self._pos = 0
+        self._chunks = None
 
     def close(self, *args, **kwargs):
         self._pos = 0
+        self._chunks = None
 
     @property
     def closed(self):
@@ -386,6 +313,7 @@ class SwiftclientStorageFile(File):
 
     def seek(self, pos):
         self._pos = pos
+        self._chunks = None
 
 
 class ThreadSafeSwiftclientStorage(SwiftclientStorage):
@@ -405,9 +333,7 @@ class ThreadSafeSwiftclientStorage(SwiftclientStorage):
 
     def _get_connection(self):
         if not hasattr(self.local_cache, "connection"):
-            public = not self.use_snet  # invert
-            connection = pyrax.connect_to_cloudfiles(region=self.region,
-                                                     public=public)
+            connection = self._get_connection()
             self.local_cache.connection = connection
 
         return self.local_cache.connection
